@@ -1,15 +1,15 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007-2009 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007-2010 -- leonerd@leonerd.org.uk
 
 package IO::Async::Loop::Glib;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.17';
-use constant API_VERSION => '0.24';
+our $VERSION = '0.19';
+use constant API_VERSION => '0.33';
 
 use base qw( IO::Async::Loop );
 
@@ -17,6 +17,8 @@ use Carp;
 
 use Glib;
 
+use Time::HiRes qw( time );
+
 =head1 NAME
 
 C<IO::Async::Loop::Glib> - use C<IO::Async> with F<Glib> or F<GTK>
@@ -184,6 +186,7 @@ sub enqueue_timer
    }
 
    my $interval = $delay * 1000; # miliseconds
+   $interval = 0 if $interval < 0; # clamp or Glib gets upset
 
    my $code = delete $params{code};
    ref $code eq "CODE" or croak "Expected 'code' to be a CODE reference";
@@ -286,7 +289,9 @@ sub loop_once
    my $context = Glib::MainContext->default;
    my $ret = $context->iteration( 1 );
 
-   Glib::Source->remove( $timerid ) unless $timed_out;
+   if( defined $timerid ) {
+      Glib::Source->remove( $timerid ) unless $timed_out;
+   }
 
    return $ret and not $timed_out ? 1 : 0;
 }
